{       selflinear.p      }
{ (c) Kenrou Adachi, 2007 }


program selflinear(input, output);

type
   points = array[1..40] of real;

const 
   RED	 = 2;
   GREEN = 3;
   BLUE	 = 4;

{$I eggxhead.p}


procedure message;
begin
   writeln('このプログラムは2次元定数係数線形自励系微分方程式:');
   writeln('        dx/dt = A * x \n の解軌道群を描画するものです。');
   writeln('計算には解公式 x(t) = x(0) + exp(t * A) * x(0) の近似式を使います。')
end; { message }


procedure inputcoeff(var s, t, u, v : real);
begin
  writeln('係数行列 A = [[s, t], [u, v]] を入力して下さい。');
  write(' s = ? ');
  readln(s);
  write(' t = ? ');
  readln(t);
  write(' u = ? ');
  readln(u);
  write(' v = ? ');
  readln(v)
end; { inputcoeff }


procedure detectcoeff(	  s, t, u, v : real;
		      var A, B, C, D : real;
			  h	     : real);
var
   s2, t2, u2, v2, s3, t3, u3, v3, s4, t4, u4, v4, s5, t5, u5, v5 : real;
begin
{ dx/dt = x + hAx + ((hA)^2/2)x + ((hA)^3/6)x + ((hA^4)/24)x + ((hA)^5/120)x }

  { A^2 }

   s2 := s * s + t * u;
   t2 := s * t + t * v;
   u2 := u * s + v * u;
   v2 := u * t + v * v;

  { A^3 }

   s3 := s2 * s + t2 * u;
   t3 := s2 * t + t2 * v;
   u3 := u2 * s + v2 * u;
   v3 := u2 * t + v2 * v; 

  { A^4 }

   s4 := s2 * s2 + t2 * u2;
   t4 := s2 * t2 + t2 * v2;
   u4 := u2 * s2 + v2 * u2;
   v4 := u2 * t2 + v2 * v2;

  { A^5 }

   s5 := s2 * s3 + t2 * u3;
   t5 := s2 * t3 + t2 * v3;
   u5 := u2 * s3 + v2 * u3;
   v5 := u2 * t3 + v2 * v3; 

  { exp(hA) - I の Maclaurin 近似 }
  
   A := h * (s + h * (s2 / 2.0 + h * (s3 / 6.0 + h * (s4 / 24.0 + h * s5 / 120.0))));
   B := h * (t + h * (t2 / 2.0 + h * (t3 / 6.0 + h * (t4 / 24.0 + h * t5 / 120.0))));
   C := h * (u + h * (u2 / 2.0 + h * (u3 / 6.0 + h * (u4 / 24.0 + h * u5 / 120.0))));
   D := h * (v + h * (v2 / 2.0 + h * (v3 / 6.0 + h * (v4 / 24.0 + h * v5 / 120.0))))

end; { detectcoeff }


procedure axes;
begin
   gsetcolor(BLUE);
   gline(0.0, 200.0, 639.0, 200.0);
   gline(320.0, 0.0, 320.0, 399.0)
end; { axes }


procedure microline(var x, y: points; a, b, h, AA, BB, CC, DD : real);
var
   j				: integer;
   x0, y0, r0, wx, wy, wx1, wy1	: real;
begin
   for j := 1 to 40 do
   begin
      { j 番目の初期点 }

      x0 := x[j];
      y0 := y[j];

      if (abs(x0) <= 2.0 * a) and (abs(y0) <= 2.0 * b) then
      begin
	 r0 := sqrt(x0 * x0 + y0 * y0);
	 
	 { x(t) = x(0) + exp(tA)x(0) }

	 if (r0 >= h) then
	 begin
	    x[j] := x0 + (AA * x0 + BB * y0);
	    wx1 := x[j];
	    y[j] := y0 + (CC * x0 + DD * y0);
	    wy1 := y[j];

	    { Window に描画範囲を割当るように座標変換する }

	    wx := 320.0 * x0 / a;
	    wy := 200.0 * y0 / b;
	    wx1 := wx1 * 320.0 / a;
	    wy1 := wy1 * 200.0 / b;

	    { 線素の描画 }

	    gline(wx + 320.0, wy + 200.0, wx1 + 320.0, wy1 + 200.0)
	 end
      end
   end
end; { selflinear }


procedure drawing(a, b, h : real; l : integer; s, t, u, v : real);
var
   j, count	  : integer;
   rnd, e1, e2, e : real;
   x, y, xx, yy	  : points;
   eh, AA, BB, CC, DD : real;
begin
   { 描画範囲に４０個の初期点をランダムに生成する。}

   randomize;
   j := 1;
   e1 := -1.0;
   while e1 <= 0.9 do
   begin
      e2 := -1.0;
      while e2 <= 0.9 do
      begin
	 rnd := random;
	 xx[j] := a * (e1 + 0.25 * rnd);
	 rnd := random;
	 yy[j] := b * (e2 + 0.4 * rnd);
	 j := j + 1;
	 e2 := e2 + 0.4
      end;
      e1 := e1 + 0.25
   end;

   { 軌道計算 routine を呼出す。}

   e := 1.0; { 先ず未来を計算 }
   repeat
      { 初期点情報を変数にコピーする。}
      for j := 1 to 40 do
      begin
	 x[j] := xx[j];
	 y[j] := yy[j];
      end;
      { 描画色の変更 }
      if e > 0 then
	 gsetcolor(GREEN)  { 未来 }
      else 
	 gsetcolor(RED);  { 過去 }

      { Ah + (Ah)^2/2 + (Ah)^3/6 + (Ah)^4/24 + (Ah)^5/120 }

      eh := e * h;
      detectcoeff(s, t, u, v, AA, BB, CC, DD, eh);

      { 軌道計算 }
      for count := 1 to l do
	 microline(x, y, a, b, h, AA, BB, CC, DD);
      e := -e; { 時間を逆転して過去を計算 }
   until e > 0
end; { drawing }


{ main }
var
   a, b, h    : real;
   s, t, u, v : real;
   m, l	      : integer;
   n	      : integer;
   dummy      : char;
begin
   { 挨拶 }

   message;

  { 係数行列の入力
       d  [x]    [ s  t ][x]
      --- [ ]  = [      ][ ]
      dt  [y]    [ u  v ][y]   }

   inputcoeff(s, t, u, v);

   { データの設定 }

   a := 6.4;
   b := 4.0;
   m := 3;
   l := 50;

   { グラフィックの初期化 }

   ginit;

   { きざみ幅の設定 }

   h := 2.0 * sqrt((a / 320.0) *(a / 320.0) +(b / 200.0) * (b / 200.0));

   { 座標軸の描画 }

   axes;

   { 繰り返し処理 }

   for n := 1 to m do
      drawing(a, b, h, l, s, t, u, v); { 線素の描画 }

   { 終了 }
   writeln('終了しました。');
   writeln('Enter key を押してください。');
   readln(dummy);
   gfinish;
end. { main }


   





