/* newp.c                      */
/* The Driver of ode2  system  */
/* (C) Kenrou Adachi, 2004     */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

/* 実行ファイル phe2 の生成コマンド */

#define COMPILE_ETPRO "egg -O2 -o phe2 phase.c func.c"


int main(void)
{
  int defun(void);
  int status;

  /* 挨拶 */

  fprintf(stderr, "正規型２階常微分方程式\n");
  fprintf(stderr, "d^2x/dt^2 = f(x, dx/dt, t)\n");
  fprintf(stderr, "の解の相空間における軌道を描画します。\n\n");
  fprintf(stderr, " y = dx/dt として、 dy / dt = f(x, y, t) と考え計算します。\n");
  fprintf(stderr, "f(x,y,t) を入力して下さい。\n");
  fprintf(stderr, "入力式には空白を入れないで下さい。\n\n");

  /* 関数入力 */

  status = defun();

  /* phe2 のコンパイル */

  if (status == EXIT_SUCCESS) {
    status = system(COMPILE_ETPRO);
  } else {
    fprintf(stderr, "入力した関数の定義式に誤りがあるか、あるいはその他の何らかの理由で、\n");
    fprintf(stderr, "func.c の生成に失敗しました。もう一度最初からやり直して下さい。\n");
    exit(EXIT_FAILURE);
  }

  /* phe2 実行 */

  if (status == EXIT_SUCCESS) {
    system("./phe2");
  } else {
    fprintf(stderr, "何らかの理由で phe2 のコンパイルに失敗しました。\n");
    fprintf(stderr, "処理を中止します。\n");
    exit(EXIT_FAILURE);
  }

  /* 終了 */
  return EXIT_SUCCESS;
}

/* func.c というファイルに 関数 fx(x,y,t) と fy(x,y,t) の定義を書き込む */

int defun(void)
{
  FILE *file;
  char *gs;

  gs = (char *)malloc(60 * sizeof(char));

  fprintf(stderr, "\nInput dy/dt = f(x, y, t)\n ? ");
  scanf("%s", gs);
  fprintf(stderr, "\n");

  file = fopen("func.c", "w");

  /* dx/dt = y */

  fprintf(file, "#include <math.h>\n\n");
  fprintf(file, "double fx(double x, double y, double t)\n");
  fprintf(file, "{\n");
  fprintf(file, "\t return y;\n");
  fprintf(file, "}\n\n\n");

  /* dy/dt = fy(x,y,t) */

  fprintf(file, "double fy(double x, double y, double t)\n");
  fprintf(file, "{\n");
  fprintf(file, "\t return %s ;\n", gs);
  fprintf(file, "}\n\n\n");

  fclose(file);
  free(gs);
  return EXIT_SUCCESS;
}

