/* news.c                       */
/* The Driver of 1st ode system */
/* (C) Kenrou Adachi, 2004      */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

/* 実行ファイル sys2 の生成コマンド */

#define COMPILE_ETPRO "egg -O2 -o sys2 phase.c func.c"


int main(void)
{
  int defun(void);
  int status;

  /* 挨拶 */

  fprintf(stderr, "連立1階常微分方程式\n");
  fprintf(stderr, "dx/dt = f(x, y, t), dy/dt = g(x,y,t)\n");
  fprintf(stderr, "の解の軌道を描画します。\n\n");
  fprintf(stderr, "f(x,y,t), g(x,y,t) を入力して下さい。\n");
  fprintf(stderr, "入力式には空白を入れないで下さい。\n\n");

  /* 関数入力 */

  status = defun();

  /* sys2 のコンパイル */

  if (status == EXIT_SUCCESS) {
    status = system(COMPILE_ETPRO);
  } else {
    fprintf(stderr, "入力した関数の定義式に誤りがあるか、あるいはその他の何らかの理由で、\n");
    fprintf(stderr, "func.c の生成に失敗しました。もう一度最初からやり直して下さい。\n");
    exit(EXIT_FAILURE);
  }

  /* sys2 実行 */

  if (status == EXIT_SUCCESS) {
    system("./sys2");
  } else {
    fprintf(stderr, "何らかの理由で sys2 のコンパイルに失敗しました。\n");
    fprintf(stderr, "処理を中止します。\n");
    exit(EXIT_FAILURE);
  }

  /* 終了 */
  return EXIT_SUCCESS;
}

/* func.c というファイルに 関数 fx(x,y,t) と fy(x,y,t) の定義を書き込む */

int defun(void)
{
  FILE *file;
  char *fs;
  char *gs;

  fs = (char *)malloc(60 * sizeof(char));
  gs = (char *)malloc(60 * sizeof(char));

  fprintf(stderr, "\nInput dy/dt = f(x, y, t)\n ? ");
  scanf("%s", fs);
  fprintf(stderr, "\n");

  fprintf(stderr, "\nInput dy/dt = g(x, y, t)\n ? ");
  scanf("%s", gs);
  fprintf(stderr, "\n");

  file = fopen("func.c", "w");

  /* dx/dt = fx(x,y,t) */

  fprintf(file, "#include <math.h>\n\n");
  fprintf(file, "double fx(double x, double y, double t)\n");
  fprintf(file, "{\n");
  fprintf(file, "\t return %s ;\n", fs);
  fprintf(file, "}\n\n\n");

  /* dy/dt = fy(x,y,t) */

  fprintf(file, "double fy(double x, double y, double t)\n");
  fprintf(file, "{\n");
  fprintf(file, "\t return %s ;\n", gs);
  fprintf(file, "}\n\n\n");

  fclose(file);
  free(gs);
  free(fs);
  return EXIT_SUCCESS;
}

