***************** difnorm2.f *******************
*     2階常微分方程式の解のグラフ                  *
*     (c) Kenrou Adachi, 2004/11/25            *
************************************************

      program difnorm2

      real    a, xo, yo, ts, te
      real    dx, dt, ox, ot
      integer nwins
      character key
      real    cpx, cpy
      common /currentp/ cpx, cpy

      call info_message

 10   continue
         call input_data(a, xo, yo, ts, te)
      if ((a.lt.0.0).or.(a.gt.160.0).or.(abs(xo).ge.abs(a))
     $              .or.(ts.ge.te)) goto 10

      call gopen(640, 400, nwins)
      call gsetbgcolor(nwins, 'black\0')
      call gclr(nwins)

      call draw_axes(nwins)

      call view_window(a, ts, te, dx, dt, ox, ot)

      call draw_curve(xo, yo, ts,te, dx, dt, ox, ot, nwins)

      write(*,*) '終了しました。'
      write(*,*) '任意のキーに続けて、エンターキーを押して下さい。'
      read(*,*) key

      call gclose(nwins)

      stop
      end
 


      subroutine info_message

      write(*,*) 'このプログラムは'
      write(*,*) '2階常微分方程式の解のグラフを描くものです。'

      return
      end



      subroutine input_data(a, xo, yo, ts, te)
      real a, xo, yo, ts, te

      write(*,*) 'x-軸の範囲 [-a, a] を入力して下さい。'
      write(*,*) 'a = ?'
      read(*,*) a
      write(*,*) 'x の初期値 xo を入力して下さい。'
      write(*,*) 'xo = ?'
      read(*,*) xo
      write(*,*) 'dx/dt の初期値 dxo を入力して下さい。'
      write(*,*) 'dxo = ?'
      read(*,*) yo
      write(*,*) '開始時刻 は 0 です。'
      ts = 0.0
      write(*,*) '終了時刻 te を入力して下さい。'
      write(*,*) 'te = ?'
      read(*,*) te

      return
      end



      subroutine view_window(a, ts, te, dx, dt, ox, ot)
      real a, ts, te, dx, dt, ox, ot

      dx = 2.0 * a
      dt = te - ts
      ox = 400.0 * a / dx
      ot = -640.0 * ts / dt

      return
      end



      subroutine draw_axes(nwins)
      integer nwins

      call newpencolor(nwins, 1)
      call line(nwins,   0.0,   0.0, 3)
      call line(nwins,   0.0, 399.0, 2)
      call line(nwins,   0.0, 200.0, 3)
      call line(nwins, 639.0, 200.0, 2)

      return
      end



      subroutine pre_draw(xo, ts, dx, dt, ox, ot, nwins)
      real    xo, ts, dx, dt, ox, ot
      integer nwins

      real oldx, oldt
      real cpx, cpy
      common /currentp/ cpx, cpy

      call newpencolor(nwins, 7)
      oldx = -400.0 * xo / dx + ox
      oldt =  640.0 * ts / dt + ot
      call line(nwins, oldt, 400.0 - oldx, 3)
      cpx = oldt
      cpy = oldx

      return
      end



      subroutine runge_kutta(xo, yo, t, h)
      real xo, yo, t, h

      real x1, x2, x3, x4, y1, y2, y3, y4

      x1 = h * fx(xo, yo, t)
      y1 = h * fy(xo, yo, t)
      x2 = h * fx(xo + x1 / 2.0, yo + y1 / 2.0, t + h / 2.0)
      y2 = h * fy(xo + x1 / 2.0, yo + y1 / 2.0, t + h / 2.0)
      x3 = h * fx(xo + x2 / 2.0, yo + y2 / 2.0, t + h / 2.0)
      y3 = h * fy(xo + x2 / 2.0, yo + y2 / 2.0, t + h / 2.0)
      x4 = h * fx(xo + x3, yo + y3, t + h)
      y4 = h * fy(xo + x3, yo + y3, t + h)

      xo = xo + (x1 + 2.0 * x2 + 2.0 * x3 + x4) / 6.0
      yo = yo + (y1 + 2.0 * y2 + 2.0 * y3 + y4) / 6.0
      t  = t + h

      return
      end



      subroutine draw_micro_line(xo, dx, dt, ox, ot, t, nwins)
      real    xo, dx, dt, ox, ot, t
      integer nwins

      real newx, newt
      real cpx, cpy
      common /currentp/ cpx, cpy

      newx = -400.0 * xo / dx + ox
      newt =  640.0 * t  / dt + ot
      call line(nwins, cpx, 400.0 - cpy, 3)
      call line(nwins, newt, 400.0 - newx, 2)
      cpx = newt
      cpy = newx

      return
      end



      subroutine draw_curve(xo, yo, ts, te, dx, dt, ox, ot, nwins)
      real    xo, yo, ts, te, dx, dt, ox, ot
      integer nwins

      real    t, h
      integer i

      h = (te - ts) / 400.0
      t = ts
      call pre_draw(xo, t, dx, dt, ox, ot, nwins)

      do 20 i = 1, 640
         call runge_kutta(xo, yo, t, h)
         call draw_micro_line(xo, dx, dt, ox, ot, t, nwins)
 20   continue

      return
      end



*      real function fx(x, y, t)
*      real x, y, t
*
*      fx = y
*
*      return
*      end
*
*
*
*      real function fy(x, y, t)
*      real x, y, t
*
*      fy = (1.0 - x * x) * y - x
*      
*      return
*      end

