****************** phase.f************************
*     2階正規型常微分方程式の相軌道                  *
*     (c) Kenrou Adachi, 2004/11/26              *
**************************************************

      program phase

      real      a, b, xo, yo, ts, te
      real      dx, dy, ox, oy
      integer   nwins
      character key
      real      cpx, cpy
      common /currentp/ cpx, cpy

      call info_message

 10   continue
         call input_data(a, b, xo, yo, ts, te)
      if ((a.le.0.0).or.(a.gt.256.0).or.(abs(xo).gt.abs(a))
     $   .or.(ts.ge.te).or.(b.le.0.0).or.(b.gt.160.0)) goto 10

      call gopen(640, 400, nwins)
      call gsetbgcolor(nwins, 'black\0')
      call gclr(nwins)

      call draw_axes(nwins)
 
      call view_window(a, b, dx, dy, ox, oy)

      call draw_curve(xo, yo, ts, te, dx, dy, ox, oy, nwins)

      write(*,*) '終了しました。'
      write(*,*) '任意のキーに続けて、エンターキーを押して下さい。'
      read(*,*) key
 
      call gclose(nwins)

      stop
      end



      subroutine info_message

      write(*,*) 'このプログラムは２階正規型常微分方程式の解の'
      write(*,*) '相空間における軌道を描画するものです。'
      write(*,*) 'すなわち, d^2x/dt^2 = f(x, dx/dt, t) の解'
      write(*,*) '(x(t),dx/dt(t))の軌道です。'

      return
      end

      subroutine input_data(a, b, xo, yo, ts, te)
      real a, b, xo, yo, ts, te

      write(*,*) ' x軸の描画範囲を入力して下さい。x : [-a, a]'
      write(*,*) ' a = ? '
      read(*,*) a
      write(*,*) ' y軸の描画範囲を入力して下さい。y : [-b, b]'
      write(*,*) ' b = ? '
      read(*,*) b
      write(*,*) ' x の初期値 xo の値を入力して下さい。'
      write(*,*) ' xo = ? '
      read(*,*) xo
      write(*,*) 'dx/dt の初期値 yo の値を入力して下さい。'
      write(*,*) ' yo = ? '
      read(*,*) yo
      write(*,*) ' 時間 t の変域を入力して下さい。t : [ts, te] です。'
      write(*,*) ' ts = ? '
      read(*,*) ts
      write(*,*) ' te = ? '
      read(*,*) te

      return
      end


      subroutine view_window(a, b, dx, dy, ox, oy)
      real a, b, dx, dy, ox, oy

      dx = 2.0 * a
      dy = 2.0 * b
      ox = 640 * a / dx
      oy = 400 * b / dy

      return
      end


      subroutine draw_axes(nwins)
      integer nwins

      call newpencolor(nwins, 1)
      call line(nwins, 320.0,   0.0, 3)
      call line(nwins, 320.0, 399.0, 2)
      call line(nwins,   0.0, 200.0, 3)
      call line(nwins, 639.0, 200.0, 2)

      return
      end


      subroutine pre_draw(xo, yo, dx, dy, ox, oy, nwins)
      real    xo, yo, dx, dy, ox, oy
      integer nwins

      real oldx, oldy
      real cpx, cpy
      common /currentp/ cpx, cpy

      call newpencolor(nwins, 7)
      oldx =   640.0 * xo / dx + ox
      oldy =  -400.0 * yo / dy + oy
      call line(nwins, oldx, 400.0 - oldy, 3)
      cpx = oldx
      cpy = oldy

      return
      end


      subroutine runge_kutta(xo, yo, t, h)
      real xo, yo, t, h

      real x1, x2, x3, x4, y1, y2, y3, y4

      x1 = h * fx(xo, yo, t)
      y1 = h * fy(xo, yo, t)
      x2 = h * fx(xo + x1 / 2.0, yo + y1 / 2.0, t + h / 2.0)
      y2 = h * fy(xo + x1 / 2.0, yo + y1 / 2.0, t + h / 2.0)
      x3 = h * fx(xo + x2 / 2.0, yo + y2 / 2.0, t + h / 2.0)
      y3 = h * fy(xo + x2 / 2.0, yo + y2 / 2.0, t + h / 2.0)
      x4 = h * fx(xo + x3, yo + y3, t + h)
      y4 = h * fy(xo + x3, yo + y3, t + h)

      xo = xo + (x1 + 2.0 * x2 + 2.0 * x3 + x4) / 6.0
      yo = yo + (y1 + 2.0 * y2 + 2.0 * y3 + y4) / 6.0
      t = t + h

      return
      end


      subroutine draw_micro_line(xo, yo, dx, dy, ox, oy, nwins)
      real    xo, yo, dx, dy, ox, oy
      integer nwins

      real newx, newy
      real cpx, cpy
      common /currentp/ cpx, cpy

      newx =  640.0 * xo /dx + ox
      newy = -400.0 * yo / dy + oy
      call line(nwins, cpx, 400.0 - cpy, 3)
      call line(nwins, newx, 400.0 - newy, 2)
      cpx = newx
      cpy = newy

      return
      end
 

      subroutine draw_curve(xo, yo, ts, te, dx, dy, ox, oy, nwins)
      real    xo, yo, ts, te, dx, dy, ox, oy
      integer nwins

      real    t, h
      integer i

      h = (te - ts) / 640.0
      t = ts
      call pre_draw(xo, yo, dx, dy, ox, oy, nwins)

      do 20 i = 1, 640
         call runge_kutta(xo, yo, t, h)
         call draw_micro_line(xo, yo, dx, dy, ox, oy, nwins)
 20   continue


      return
      end


*      real function fx(x, y, t)
*      real x, y, t
*
*      fx = y
*
*      return
*      end
*
*
*      real function fy(x, y, t)
*      real x, y, t
*
*      fy = (1.0 - x * x) * y - x
*
*      return
*      end


